/***** global.c ***************************************************
 * Description: Global alignment of protein or DNA sequences.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 19:53:38 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "eprintf.h"
#include "interface.h"
#include "sequenceData.h"
#include "alignment.h"
#include "stringUtil.h"

int main(int argc, char *argv[]){
  char *version;
  Args args; 
  Sequence *s1 = NULL, *s2 = NULL;
  Alignment *al;
  SubstitutionMatrix *sm = NULL;
  FILE *fpq, *fps, *fpo, *fpm = NULL;

  setprogname2("gal");
  setAlignmentMode(GLOBAL);

  version = VERSION;
  args = getArgs(argc, argv);
  
  if(args.h == 1){
    printUsage(version);
    return 0; 
  }else if(args.v == 1){
    printSplash(version);
    return 0;
  }else if(args.e == 1){
    printUsage(version);
    return -1;
  }

  fpq = efopen(args.i,"r");
  s1 = getNextSequence(fpq);
  resetSequenceReader();
  fclose(fpq);
  if(args.j)
    fps = efopen(args.j,"r");
  else
    fps = stdin;

  if(args.m == NULL){
    sm = constructDnaScoreMatrix(sm, args.A, args.I);
  }else{
    fpm = efopen(args.m, "r");
    sm = readSubstitutionMatrix(sm, fpm);
  }
  if(args.m)
    fclose(fpm);

  if(args.o == NULL)
    fpo = stdout;
  else
    fpo = efopen(args.o,"w");

  while((s2 = getNextSequence2(fps,s2)) != NULL){
    al = global(s1->seq,s2->seq,args.O,args.E,sm);
    if(al){
      al->name1 = strdup2(s1->id);
      al->name2 = strdup2(s2->id);
      if(args.p)
	al->strand = NULL;
      else
	al->strand = "Plus / Plus";
      printAlignment(al,fpo,sm,args);
    }
    free(al);
  }
  if(args.o)
    fclose(fpo);
  if(args.j)
    fclose(fps);
  return 0;
}
