/***** globalAlignment.c ******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 12:07:53 2004.
 *****************************************************************/
#include <stdlib.h>
#include "string.h"
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"
#include "limits.h"

void *initializeGlobalDpm(int open, int extend);
DpMatrix *dpm = NULL;

/* optimal global alignment of two sequences */
Alignment *global(char *seq1, char *seq2, float open, float extend, SubstitutionMatrix *sm){
  int i, j = 0, m, n;
  float a, b, c, x, y;
  Alignment *al = NULL;
  MatrixPosition *pos;

  open += extend;       /* gap of length 1 has weight even if gap opening is set to zero */
  n = strlen(seq1) + 1;
  m = strlen(seq2) + 1;
  dpm = generateDpMatrix(m, n, dpm);
  dpm->hseq = seq1;
  dpm->vseq = seq2;
  dpm = initializeGlobalDpm(open, extend);
  /* fill in remainder of matrix */
  for(i=1; i<m; i++){
    for(j=1; j<n; j++){
      dpm->mat[i][j][0].isVisited = 0;
      dpm->mat[i][j][1].isVisited = 0;
      dpm->mat[i][j][2].isVisited = 0;
      a = dpm->mat[i-1][j][0].s + open;
      b = dpm->mat[i-1][j][1].s + extend;
      if(a > b){
	dpm->mat[i][j][1].s = a;
	dpm->mat[i][j][1].next = 0;
      }else{
	dpm->mat[i][j][1].s = b;
	dpm->mat[i][j][1].next = 1;
      }
      /* fill in c */
      a = dpm->mat[i][j-1][0].s + open;
      c = dpm->mat[i][j-1][2].s + extend;
      if(a > c){
	dpm->mat[i][j][2].s = a;
	dpm->mat[i][j][2].next = 0;
      }else{
	dpm->mat[i][j][2].s = c;
	dpm->mat[i][j][2].next = 2;
      }
      /* fill in a */ 
      a = dpm->mat[i-1][j-1][0].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      b = dpm->mat[i-1][j-1][1].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      c = dpm->mat[i-1][j-1][2].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      y = (a > b) ? a : b;
      x = (c > y) ? c : y;
      if(a == x){
	dpm->mat[i][j][0].s = a;
	dpm->mat[i][j][0].next = 0;
      }else if(b == x){
	dpm->mat[i][j][0].s = b;
	dpm->mat[i][j][0].next = 1;
      }else if(c == x){
	dpm->mat[i][j][0].s = c;
	dpm->mat[i][j][0].next = 2;
      }
    }
  }
  /* writeDpm(dpm); */
  y = (dpm->mat[m-1][n-1][1].s > dpm->mat[m-1][n-1][2].s) ? dpm->mat[m-1][n-1][1].s : dpm->mat[m-1][n-1][2].s;
  x = (y > dpm->mat[m-1][n-1][0].s) ? y : dpm->mat[m-1][n-1][0].s;
  if(x == dpm->mat[m-1][n-1][0].s)
    i = 0;
  else if(x == dpm->mat[m-1][n-1][1].s)
    i = 1;
  else 
    i = 2;
  al = (Alignment *)emalloc(sizeof(Alignment));
  al->head1 = NULL;
  al->head2 = NULL;
  pos = (MatrixPosition *)emalloc(sizeof(MatrixPosition));
  pos->i = m-1;
  pos->j = n-1;
  pos->k = i;
  al = traceBack(dpm,al,pos,pos->j,pos->i,INT_MAX);
  freeDpm(dpm);
  free(pos);
  dpm = NULL;
  return al;
}

void *initializeGlobalDpm(int open, int extend){
  int i, m, n;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
 
  dpm->mat[0][0][0].s = 0;
  dpm->mat[0][0][1].s = 0;
  dpm->mat[0][0][2].s = 0;
  dpm->mat[0][0][0].next = -1;
  dpm->mat[0][0][1].next = -1;
  dpm->mat[0][0][2].next = -1;
  dpm->mat[0][0][0].isVisited = 0;
  dpm->mat[0][0][1].isVisited = 0;
  dpm->mat[0][0][2].isVisited = 0;
  /* initialize first column */
  for(i=1;i<m;i++){
    dpm->mat[i][0][0].s = INT_MIN/2;
    dpm->mat[i][0][1].s = open + (i-1)*extend;
    dpm->mat[i][0][2].s = INT_MIN/2;
    dpm->mat[i][0][0].next = -1;
    dpm->mat[i][0][1].next = 1;
    dpm->mat[i][0][2].next = -1;
    dpm->mat[i][0][0].isVisited = 0;
    dpm->mat[i][0][1].isVisited = 0;
    dpm->mat[i][0][2].isVisited = 0;
  }
  /* initialize first row */
  for(i=1;i<n;i++){
    dpm->mat[0][i][0].s = INT_MIN/2;
    dpm->mat[0][i][1].s = INT_MIN/2;
    dpm->mat[0][i][2].s = open + (i-1)*extend;
    dpm->mat[0][i][0].next = -1;
    dpm->mat[0][i][1].next = -1;
    dpm->mat[0][i][2].next = 2;
    dpm->mat[0][i][0].isVisited = 0;
    dpm->mat[0][i][1].isVisited = 0;
    dpm->mat[0][i][2].isVisited = 0;
  }
  return dpm;
}

