/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "interface.h"
#include "jabberwokky.h"

int lineLength = 70;
int npref = 2;
int maxWords = 10000;

Args getArgs(int argc, char *argv[]){
  Args args;
  char c;
  int arg;
  
  args.i = NULL;
  args.o = NULL;
  args.s = 0;
  args.l = lineLength;
  args.m = maxWords; 
  args.p = npref;
  args.h = 0;
  args.e = 0;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'i':                           /* input file */
	args.i = argv[++arg];
	break;
      case 'o':                           /* output file */
	args.o = argv[++arg];
	break;
      case 's':                           /* seed for random number generator */
	args.s = atoi(argv[++arg]);
	break;
      case 'l':                           /* seed for random number generator */
	args.l = atoi(argv[++arg]);
	break;
      case 'm':                           /* maximum number of words generated */
	args.m = atoi(argv[++arg]);
	break;
      case 'p':                           /* number of prefix words */
	args.p = atoi(argv[++arg]);
	break;
      case 'h':                           /* print help */
	args.h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args.e = 1;
	return args;
      }
      arg++;
  }
  return args;
}


void printUsage(char *version){
  printf("jabbberwokky version %s\n", version);
  printf("purpose: wittify prose; for suitable input see: www.gutenberg.org\n");
  printf("algorithms: Kernighan, B. W. and Pike, R. (1999). The Practice\n");
  printf("            of Programming. Addision Wesley; chapter 3.\n");
  printf("author: Bernhard Haubold\n");    
  printf("arguments:  \n");
  printf("\t[-i input file; default: stdin]\n");
  printf("\t[-o output file; default: stdout]\n");
  printf("\t[-p number of words in prefix; default: %d]\n",npref);
  printf("\t[-m maximum number of words generated; default: %d]\n",maxWords);
  printf("\t[-s seed for random number generator; default: system clock]\n");
  printf("\t[-l length of output line; default: %d]\n",lineLength);	      
  printf("\t[-h print this help message]\n");
}
