/***** kerrorAlignment.c ******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 12:07:53 2004.
 *****************************************************************/
#include <string.h>
#include <limits.h>
#include <stdlib.h>
#include "stringUtil.h"
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"

DpMatrix *initializeLocalDpm(DpMatrix *dpm);
MatrixPosition *findKerrorMax(DpMatrix *dpm); 

/* kerror alignment of two sequences */
Alignment *alignKerror(char *subject, char *query, int lb, int rb, SubstitutionMatrix *sm, Args args){
  int a, b, c = 0, i, j, m, n, x;
  float open, extend;
  MatrixPosition *pos;

  open = args.O + args.E;
  extend = args.E;
  
  open += extend;       /* gap of length 1 has weight even if gap opening is set to zero */
  DpMatrix *dpm = NULL;
  Alignment *al = NULL;

  n = strlen(query) + 1;
  m = rb - lb + 2;

  dpm = generateDpMatrix(m, n, dpm);
  dpm->vseq = emalloc(m*sizeof(char));
  j = 0;
  for(i=0; i<m-1; i++)
    dpm->vseq[j++] = subject[lb+i];
  dpm->vseq[j] = '\0';
  dpm->hseq = strdup2(query);
  dpm = initializeLocalDpm(dpm);
  for(i=1; i<m; i++){
    for(j=1; j<n; j++){
      dpm->mat[i-1][j][0].isVisited = 0;
      dpm->mat[i-1][j][1].isVisited = 0;
      dpm->mat[i-1][j][2].isVisited = 0;
      a = dpm->mat[i-1][j][0].s + open;
      b = dpm->mat[i-1][j][1].s + extend;
      if(a > b){
	dpm->mat[i][j][1].s = a;
	dpm->mat[i][j][1].next = 0;
      }else{
	dpm->mat[i][j][1].s = b;
	dpm->mat[i][j][1].next = 1;
      }
      /* fill in c */
      a = dpm->mat[i][j-1][0].s + open;
      c = dpm->mat[i][j-1][2].s + extend;
      if(a > c){
	dpm->mat[i][j][2].s = a;
	dpm->mat[i][j][2].next = 0;
      }else{
	dpm->mat[i][j][2].s = c;
	dpm->mat[i][j][2].next = 2;
      }
      /* fill in a */
      a = dpm->mat[i-1][j-1][0].s + sm->mat[sm->dic[(int)dpm->vseq[i-1]]][sm->dic[(int)dpm->hseq[j-1]]];
      b = dpm->mat[i-1][j-1][1].s + sm->mat[sm->dic[(int)dpm->vseq[i-1]]][sm->dic[(int)dpm->hseq[j-1]]];
      c = dpm->mat[i-1][j-1][2].s + sm->mat[sm->dic[(int)dpm->vseq[i-1]]][sm->dic[(int)dpm->hseq[j-1]]];
      x = max(a,max(b,c));
      if(a == x){
	dpm->mat[i][j][0].s = a;
	dpm->mat[i][j][0].next = 0;
      }else if(b == x){
	dpm->mat[i][j][0].s = b;
	dpm->mat[i][j][0].next = 1;
      }else if(c == x){
	dpm->mat[i][j][0].s = c;
	dpm->mat[i][j][0].next = 2;
      }
    }
  }
  al = (Alignment *)emalloc(sizeof(Alignment));
  al->head1 = NULL;
  al->head2 = NULL;
  pos = findKerrorMax(dpm);

  if(dpm->mat[pos->i][pos->j][pos->k].s >= strlen(query)/2){ /* very rough heuristic */
/*     al = traceBack(dpm,al,pos,pos->j,rb-(m-pos->i)+2,args.k); */
    al = traceBack(dpm,al,pos,pos->j,rb-(m-pos->i)+2,args.k);
  }else{
    al = NULL;
  }
  freeDpm(dpm);
  return al;
}

DpMatrix *initializeLocalDpm(DpMatrix *dpm){
  int i, m, n;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;

  dpm->mat[0][0][0].s = 0;
  dpm->mat[0][0][1].s = 0;
  dpm->mat[0][0][2].s = 0;
  dpm->mat[0][0][0].next = -1;
  dpm->mat[0][0][1].next = -1;
  dpm->mat[0][0][2].next = -1;
  /* initialize first column */
  for(i=1;i<m;i++){
    dpm->mat[i][0][0].s = INT_MIN/2;
    dpm->mat[i][0][1].s = 0;
    dpm->mat[i][0][2].s = INT_MIN/2;
    dpm->mat[i][0][0].next = -1;
    dpm->mat[i][0][1].next = -1;
    dpm->mat[i][0][2].next = -1;
  }
  /* initialize first row */
  for(i=1;i<n;i++){
    dpm->mat[0][i][0].s = INT_MIN/2;
    dpm->mat[0][i][1].s = INT_MIN/2;
    dpm->mat[0][i][2].s = 0;
    dpm->mat[0][i][0].next = -1;
    dpm->mat[0][i][1].next = -1;
    dpm->mat[0][i][2].next = -1;
  }
  return dpm;
}

MatrixPosition *findKerrorMax(DpMatrix *dpm){
  int i, k, m, n;
  int maximum;
  MatrixPosition *pos;

  pos = (MatrixPosition *)emalloc(sizeof(MatrixPosition));
  m = strlen(dpm->vseq);
  n = strlen(dpm->hseq);
  maximum = INT_MIN;
  /* check last column */
  pos->j = n;
  for(i=0;i<=m;i++){
    for(k=0;k<3;k++){
      if(dpm->mat[i][n][k].s > maximum){
	pos->i = i;
	pos->k = k;
	maximum = dpm->mat[i][n][k].s;
      }
    }
  }
  return pos;
}
