/***** ktree.h ****************************************************
 * Description: header file for keyword tree construction.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Thu Dec 16 10:14:57 2004.
 *****************************************************************/
/* data structures */
/* data associated with a node in the keyword tree */
typedef struct kTreeData{
  int id;                 /* sequence identifier */ 
  int pos;                /* position in query */
} KTreeData;
/* result of search */
typedef struct kTreeResult{
  struct kTreeResult *next;
  KTreeData *data;
  int pos;                  /* position in text */
} KTreeResult;
/* node in keyword tree */
typedef struct node{
  int depth;           /* number of characters on path from root to node */
  char c;              /* character labelling incoming edge */
  struct node *fail;   /* failure link */
  struct node *out;    /* output link */
  struct node *child;
  struct node *sibling;
  KTreeData *data;     /* pointer for storing data associated with a node in the keyword tree */
} KNode; 

/* public function prototypes */
void ktree_addString(char *str, KTreeData *listElement, KNode *root);
KNode *ktree_constructRoot();
void ktree_preprocessTree(KNode *root);
KTreeData *ktree_constructDataItem();
KTreeResult *ktree_search(char *text, KNode *root);
