/***** local.c ****************************************************
 * Description: Local alignment of protein or DNA sequences.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Jan 10 22:54:48 2005.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "eprintf.h"
#include "interface.h"
#include "sequenceData.h"
#include "alignment.h"
#include "stringUtil.h"

int main(int argc, char *argv[]){
  char *version;
  Args args; 
  Sequence *query = NULL, *subject = NULL;
  Alignment *al;
  SubstitutionMatrix *sm = NULL;
  FILE *fp, *fps, *fpq;
  int c = 0;

  setprogname2("lal");
  setAlignmentMode(LOCAL);

  version = VERSION;
  args = getArgs(argc, argv);
  
  if(args.h == 1){
    printUsage(version);
    return 0; 
  }else if(args.v == 1){
    printSplash(version);
    return 0;
  }else if(args.e == 1){
    printUsage(version);
    return -1;
  }

  fpq = efopen(args.i,"r");
  if(args.j != NULL)
    fps = efopen(args.j,"r");
  else
    fps = stdin;

  if(args.m == NULL){
    sm = constructDnaScoreMatrix(sm, args.A, args.I);
  }else{
    fp = efopen(args.m, "r");
    sm = readSubstitutionMatrix(sm, fp);
  }

  if(args.o == NULL)
    fp = stdout;
  else
    fp = efopen(args.o,"w");

  query = getNextSequence2(fpq,query);
  while((subject = getNextSequence2(fps,subject)) != NULL){
    al = local(query->seq,subject->seq,args.O,args.E,sm,args);
    c = 0;
    while(al != NULL && (args.n > c++ || al->score >= args.s)){
      al->name1 = strdup2(query->id);
      al->name2 = strdup2(subject->id);
      if(args.p)
	al->strand = NULL;
      else
	al->strand = "Plus / Plus";
      printAlignment(al,fp,sm,args);
      al = al->next;
    }
  }
  if(args.i != NULL)
    fclose(fps);
  fclose(fpq);
  return 0;
}
