/***** localAlignment.c *******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 12:07:53 2004.
 *****************************************************************/
#include <string.h>
#include <limits.h>
#include <stdlib.h>
#include "eprintf.h"
#include "interface.h"
#include "alignment.h"

DpMatrix *initializeLocalDpm(DpMatrix *dpm);
Alignment *getAlignments(DpMatrix *dpm, Args args);
MatrixPosition *findLocalMax(DpMatrix *dpm, MatrixPosition *pos);
int ccmp(const void *a, const void *b);
DpMatrix *dpmGlobal = NULL;

/* optimal overlap alignment of two sequences */
Alignment *local(char *seq1, char *seq2, float open, float extend, SubstitutionMatrix *sm, Args args){
  int a, b, c = 0, i, j = 0, k, m, n, x;
  
  open += extend;       /* gap of length 1 has weight even if gap opening is set to zero */
  Alignment *al = NULL;

  n = strlen(seq1) + 1;
  m = strlen(seq2) + 1;
  
  dpmGlobal = generateDpMatrix(m, n, dpmGlobal);
  dpmGlobal->hseq = seq1;
  dpmGlobal->vseq = seq2;
  dpmGlobal = initializeLocalDpm(dpmGlobal);
  /* fill in remainder of matrix */
  for(i=1; i<m; i++){
    for(j=1; j<n; j++){
      for(k=0;k<3;k++)
	dpmGlobal->mat[i][j][k].isVisited = 0;
      a = dpmGlobal->mat[i-1][j][0].s + open;
      b = dpmGlobal->mat[i-1][j][1].s + extend;
      x = max(a,max(b,0));
      if(x == 0){
	dpmGlobal->mat[i][j][1].s = 0;
	dpmGlobal->mat[i][j][1].next = -1;
      }else if(a == x){
	dpmGlobal->mat[i][j][1].s = a;
	dpmGlobal->mat[i][j][1].next = 0;
      }else{
	dpmGlobal->mat[i][j][1].s = b;
	dpmGlobal->mat[i][j][1].next = 1;
      }
      /* fill in c */
      a = dpmGlobal->mat[i][j-1][0].s + open;
      c = dpmGlobal->mat[i][j-1][2].s + extend;
      x = max(a,max(c,0));
      if(x == 0){
	dpmGlobal->mat[i][j][2].s = 0;
	dpmGlobal->mat[i][j][2].next = -1;
      }
      if(a > c){
	dpmGlobal->mat[i][j][2].s = a;
	dpmGlobal->mat[i][j][2].next = 0;
      }else{
	dpmGlobal->mat[i][j][2].s = c;
	dpmGlobal->mat[i][j][2].next = 2;
      }
      /* fill in a */
      a = dpmGlobal->mat[i-1][j-1][0].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      b = dpmGlobal->mat[i-1][j-1][1].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      c = dpmGlobal->mat[i-1][j-1][2].s + sm->mat[sm->dic[(int)seq2[i-1]]][sm->dic[(int)seq1[j-1]]];
      x = max(a,max(max(b,c),0));
      if(x == 0){
	dpmGlobal->mat[i][j][0].s = 0;
	dpmGlobal->mat[i][j][0].s = -1;
      }else if(a == x){
	dpmGlobal->mat[i][j][0].s = a;
	dpmGlobal->mat[i][j][0].next = 0;
      }else if(b == x){
	dpmGlobal->mat[i][j][0].s = b;
	dpmGlobal->mat[i][j][0].next = 1;
      }else if(c == x){
	dpmGlobal->mat[i][j][0].s = c;
	dpmGlobal->mat[i][j][0].next = 2;
      }
    }
  }
  //writeDpm(dpmGlobal);
  al = getAlignments(dpmGlobal, args);
  freeDpm(dpmGlobal);
  return al;
}

Alignment *getAlignments(DpMatrix *dpm, Args args){
  Alignment *al = NULL;
  Alignment *alHead = NULL;
  Alignment *alTail = NULL;
  MatrixPosition *pos;
  int i, j, k, m, n, arraySize, numAl;
  DpCell *cellArray = NULL, *cp;
  long c = 0;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;

  pos = (MatrixPosition *)emalloc(sizeof(MatrixPosition));
  alHead = (Alignment *)emalloc(sizeof(Alignment));
  alHead->head1 = NULL;
  alHead->head2 = NULL;
  arraySize = 3*m*n;  
  numAl = 0;

  if(args.n > 1){
    cellArray = (DpCell *)emalloc(arraySize*sizeof(DpCell));
    cp = cellArray;
    
    for(i=0;i<m;i++){
      for(j=0;j<n;j++){
	for(k=0;k<3;k++){
	  *cp++ = dpm->mat[i][j][k];
	}
      }
    }
    qsort(cellArray,arraySize+1,sizeof(DpCell),ccmp);
    cellArray = cellArray+arraySize;
    c = 0;
    pos->k = cellArray->id % 3;
    pos->i = (cellArray->id - pos->k)/3/n;
    pos->j = (cellArray->id - pos->k)/3 % n;
  }else{
    pos = findLocalMax(dpm, pos);
  }
  if(pos != NULL)
    alHead = traceBack(dpm,alHead,pos,pos->j,pos->i,INT_MAX);
  if(alHead == NULL || alHead->score <= args.s || args.n < 1){
    return NULL;
  }else{
    c++;
    numAl++;
    alHead->next = NULL;
    cellArray--;
  }
  alTail = alHead;
  while(numAl < args.n && arraySize > c && cellArray->s > args.s &&  !cellArray->isVisited){
    al = (Alignment *)emalloc(sizeof(Alignment));
    al->head1 = NULL;
    al->head2 = NULL;
    pos->k = cellArray->id % 3;
    pos->i = (cellArray->id - pos->k)/3/n;
    pos->j = (cellArray->id - pos->k)/3 % n;
    if(pos == NULL)
      break;
    al = traceBack(dpm,al,pos,pos->j,pos->i,INT_MAX);
    if(al != NULL){
      al->next = NULL;
      alTail->next = al;
      alTail = alTail->next;
      numAl++;
    }
    c++;
    cellArray--;
  }
  return alHead;
}
/* ccmp: compare cells of dynamic programming matrix w. r. t. score */
int ccmp(const void *p1, const void *p2){
  DpCell c1, c2;

  c1 = *(DpCell *)p1;
  c2 = *(DpCell *)p2;
  if(c1.s < c2.s)
    return -1;
  else if(c1.s > c2.s)
    return 1;
  else
    return 0;
}

DpMatrix *initializeLocalDpm(DpMatrix *dpm){
  int i, k, m, n;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
  for(i=0;i<3;i++){
    dpm->mat[0][0][i].s = 0;
    dpm->mat[0][0][i].next = -1;
  }
  /* initialize first column */
  for(i=1;i<m;i++){
    dpm->mat[i][0][0].s = INT_MIN/2;
    dpm->mat[i][0][1].s = 0;
    dpm->mat[i][0][2].s = INT_MIN/2;
    dpm->mat[i][0][0].next = -1;
    dpm->mat[i][0][1].next = -1;
    dpm->mat[i][0][2].next = -1;
    for(k=0;k<3;k++)
      dpm->mat[i][0][k].isVisited = 0;
  }
  /* initialize first row */
  for(i=1;i<n;i++){
    dpm->mat[0][i][0].s = INT_MIN/2;
    dpm->mat[0][i][1].s = INT_MIN/2;
    dpm->mat[0][i][2].s = 0;
    dpm->mat[0][i][0].next = -1;
    dpm->mat[0][i][1].next = -1;
    dpm->mat[0][i][2].next = -1;
    for(k=0;k<3;k++)
      dpm->mat[0][i][k].isVisited = 0;
  }
  return dpm;
}

MatrixPosition *findLocalMax(DpMatrix *dpm, MatrixPosition *pos){
  int i, j, k, m, n;
  int maxVal = INT_MIN;
  int found = 0;

  m = strlen(dpm->vseq)+1;
  n = strlen(dpm->hseq)+1;
  for(i=0;i<m;i++){
    for(j=0;j<n;j++){
      for(k=0;k<3;k++){
	if(dpm->mat[i][j][k].s > maxVal && !dpm->mat[i][j][k].isVisited){
	  pos->i = i;
	  pos->j = j;
	  pos->k = k;
	  maxVal = dpm->mat[i][j][k].s;
	  found = 1;
	}
      }
    }
  }
  if(found)
    return pos;
  else 
    return NULL;
}
