#!/usr/bin/awk -f
# Date: June 15, 2017 (version 0.1)
#       July 12, 2017 (version 0.2)
BEGIN{
    program = "ranDot.awk"
    version = "0.2"
    if(h){
	printUsage(program)
    }
    if(v){
	printVersion(program, version)
	exit 
    }
    if(!n){
	printUsage(program)
    }
    if(!p)
	p = 0.1
    if(seed)
	srand(seed)
    else
	srand()
    # generate random edges
    for(i=0; i<n; i++){
	for(j=0; j<n; j++){
	    if(i != j){
		r = rand()
		if(r <= p){
		    mat[i, j] = 1
		    used[i] = 1
		    used[j] = 1
		}else
		    mat[i, j] = 0
	    }
	}
    }
    # draw graph
    printf "graph G {\n"
    # draw nodes connected by at least one edge
    for(i=0; i<n; i++){
	for(j=0; j<n; j++){
	    if(mat[i, j]){
		printf "\t%d -- %d", i+1, j+1
		mat[i, j] = 0
		if(dir){
		    if(mat[j, i]){
			printf " [dir=both]"
			mat[j, i] = 0
		    }else
			printf " [dir=forward]"
		}
		printf "\n"
	    }
	}
    }
    # draw unconnected edges
    for(i=0; i<n; i++)
	if(!used[i])
	    printf "\t%d\n", i
    printf "}\n"
}

function printUsage(prog){
    	printf "Usage: awk -f %s -v n=<n> [-v p=<p> -v dir=0|1 -v seed=<seed> -v h=1 -v v=1]\n", prog
	printf "Purpose: Draw random graph in dot language for rendering with GraphViz\n"
	printf "Options:\n"
	print "\tn: number of nodes"
	print "\t[p: edge probability (default: 0.01)]"
	print "\t[dir: directed edges (default: undirected)]"
	print "\t[seed: seed for random number generator (default: system)]"
	print "\t[h: help]"
	print "\t[v: version]"
	exit
}

function printVersion(prog, ver){
    	printf "%s %s\n", prog, ver
	printf "Written by Bernhard Haubold.\n"
	printf "Distributed under the GNU General Public License.\n"
	printf "Please send bug reports to haubold@evolbio.mpg.de\n"
	exit
}
