#!/usr/bin/awk -f
# Reference: Knuth, D. E. (1998). The Art of Computer Programming.
#   Third edition, vol. 2, p. 137. Addison Wesley.
# Author: Bernhard Haubold, haubold@evolbio.mpg.de
# Date: June 20, 2017 (version 0.1)
#       July 12, 2017 (version 0.2)
BEGIN{
    program = "rpois.awk"
    version = "0.2"
    if(h){
	printUsage(program)
    }
    if(v){
	printVersion(program, version)
    }
    if(!lambda){
	printUsage(program)
    }
    if(seed)
	srand(seed)
    else
	srand()
    r = exp(-lambda)
    k = 0
    s = 1
    u = rand()
    s *= u
    while(s >= r){
	u = rand()
	s *= u
	k++
    }
    print k	
}

function printUsage(prog){
    	printf "Usage: awk -f %s -v lambda=<lambda> [-v h=1 -v v=1]\n", prog
	printf "Purpose: Draw Poisson-distributed random number.\n"
	printf "Options:\n"
	printf "\t[lambda\tmean]\n"
	printf "\t[h\thelp]\n"
	printf "\t[v\tversion]\n"
	exit
}

function printVersion(prog, ver){
    	printf "%s %s\n", prog, ver
	printf "Written by Bernhard Haubold.\n"
	printf "Distributed under the GNU General Public License.\n"
	printf "Please send bug reports to haubold@evolbio.mpg.de\n"
	exit
}
