/***** decode.c ***********************************
 * Description: BWT-decode.
 * Reference: Donald Adjeroh, Timorhy Bell, and
 *    Amar Mukherjee (2008). The Burrows-Wheeler 
 *    Transform; Data Compression, Suffix Arrays, 
 *    and Pattern Matching. Springer, p. 26.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Mon Mar  5 12:22:11 2012
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "eprintf.h"
#include "sequenceData.h"
#include "interface.h"

void decode(Args *args, Sequence *seq){
  int i, j, n, sum, dictSize, start, end;
  /* let F be the first and L the last column in the sorted array of rotations */
  int *K;  /* K[i]: count of character seq[i] */
  int *C;  /* C[i]: occurrences of character seq[i] before position i in L */
  int *M;  /* M[i]: F[M[i]]: first time character seq[i] occurs in F */
  char *Q; /* output string */

  dictSize = 256;
  n = seq->len - 1;
  K = (int *)emalloc(sizeof(int)*dictSize);
  M = (int *)emalloc(sizeof(int)*dictSize);
  C = (int *)emalloc(sizeof(int)*n);
  Q = (char *)emalloc(sizeof(char)*n);
  /* initialize K */
  for(i=0;i<dictSize;i++)
    K[i] = 0;
  /* count characters in K */
  /* record number of previous appearances of character in C */
  for(i=0;i<n;i++){
    C[i] = K[(int)seq->seq[i]];
    K[(int)seq->seq[i]]++;
  }
  /* first occurrence of character in F */
  sum = 0;
  for(i=0;i<dictSize;i++){
    M[i] = sum;
    sum += K[i];
  }
  /* look for starting character */
  for(i=0;i<n;i++)
    if(seq->seq[i] == BORDER){
      break;
    }
  for(j=n-1;j>-1;j--){
    Q[j] = seq->seq[i];
    i = C[i] + M[(int)seq->seq[i]];
  }
  sum = 0;
  if(args->D){
    printf("P: ");
    for(i=0;i<n;i++)
      printf("%d",(i+1)%10);
    printf("\nS: ");
    for(i=0;i<n;i++)
      printf("%c",seq->seq[i]);
    printf("\nC: ");
    for(i=0;i<n;i++)
      printf("%d",C[i]);
    printf("\nK:\n");
    for(i=0;i<dictSize;i++)
      if(K[i]>0)
	printf("%c\t%d\n",i,K[i]);
    printf("M:\n");
    for(i=0;i<dictSize;i++)
      if(K[i]>0)
	printf("%c\t%d\n",i,M[i]+1);
    printf("P: ");
    for(i=0;i<n;i++)
      printf("%d",(i+1)%10);
    printf("\nS: ");
  }else{
    printf("%s - decoded\n",seq->headers[0]);
  }
  if(args->p){
    start = args->p[0] > -1 ? args->p[0] : 0;
    end = args->p[1] + 2 < n ? args->p[1] + 2 : n;
  }else{
    start = 0;
    end = n;
  }
  sum = 0;
  for(i=start;i<end-1;i++){
    printf("%c",Q[i]);
    sum++;
    if(sum == 70){
      printf("\n");
      sum = 0;
    }
  }
  if(args->D){
    sum++;
    printf("%c",Q[i]);
  }
  if(sum)
    printf("\n");
  
  free(C);
  free(K);
  free(M);
  free(Q);
}
