/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "sequenceData.h"
#include "stringUtil.h"
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char **posArray;
  int num;
  char *optString = "hvDdrcs:p:l:";

  posArray = (char  **)emalloc(2*sizeof(char *));

  args = (Args *)emalloc(sizeof(Args));
  args->D = 0;
  args->d = 0;
  args->s = BORDER;
  args->p = NULL;
  args->c = 0;
  args->l = DEFAULT_L;
  args->r = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 's':
      args->s = optarg[0];              /* sentinel */
      break;
    case 'p':
      args->p = (int *)emalloc(2*sizeof(int));
      split(optarg,"-",posArray,&num);  /* positions to be printed */
      args->p[0] = atoi(posArray[0])-1;
      args->p[1] = atoi(posArray[1])-1;
      break;
    case 'c':
      args->c = 1;
      break;
    case 'r':                           /* print runs? */
      args->r = 1;
      break;
    case 'l':
      args->l = atoi(optarg);
      break;
    case 'd':                           /* decode */
      args->d = 1;
      break;
    case 'D':                           /* demo mode */
      args->D = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  free(posArray);
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("Compute Burrows-Wheeler Transform\n");
  printf("bwt hamlet.fasta\n");
  printf("Options:\n");
  printf("\t[-d decode; default: encode]\n");
  printf("\t[-p START-END; print transform or decoded string from START to END; default print all]\n");
  printf("\t[-r print transform as character runs; default: print plain transform]\n");
  printf("\t[-c print context; default: print bwt]\n");
  printf("\t[-D demo mode]\n");
  printf("\t[-s sentinel character in encoded text; default: %c]\n",args->s);
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
