/***** cchar.c ************************************
 * Description: Count characters in FASTA files
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Jul 19 22:06:24 2004.
 **************************************************/
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include "cchar.h"
#include "interface.h"
#include "eprintf.h"

double totalChar(double *ch){
  int i;
  double s;

  s = 0;
  for(i=0;i<NUMCHAR;i++)
    s += ch[i];
  return s;
}

void printCchar(double *ch){
  int i;
  long s;

  printf("# Char\tCount\tFraction\n");
  s = (long)totalChar(ch);
  for(i=0;i<NUMCHAR;i++)
    if(ch[i]>0)
      printf("%c\t%.0f\t%f\n",i,ch[i],ch[i]/(double)s);
}

int scanFile(Args *args, FILE *fp, short *dic, char *headerBuf, double *chs){
  int i, j;
  int numSeq;
  ssize_t c;
  size_t len = 0;
  double ch[NUMCHAR]; /* character count per sequence */
  char *line;

  /* initialize character array */
  for(i=0;i<NUMCHAR;i++){
    ch[i] = 0.0;   
    chs[i] = 0.0;  /* total sum of characters */
  }
  numSeq = 0;
  line = 0;
  while((c = getline(&line, &len, fp)) != -1){
      if(line[0] == '>'){
	numSeq++;
	if(args->s && numSeq > 1){
	  if(args->E)
	    printf("%s: %ld\n", headerBuf, (long)totalChar(ch));
	  else
	    printf("%s: %g\n", headerBuf, totalChar(ch));
	  printCchar(ch);
	}
	headerBuf = strncpy(headerBuf, line, args->b);
	headerBuf[c-1] = '\0';    /* remove newline character */
	for(j=0; j<NUMCHAR; j++){
	  chs[j] += ch[j];
	  ch[j] = 0;
	}
      }else{
	for(i=0; i<c; i++){
	  if(dic[(int)line[i]])
	    ch[(int)line[i]]++;
	}
    }
  }
  if(args->s){
    if(args->E)
      printf("%s: %ld\n",headerBuf, (long)totalChar(ch));
    else
      printf("%s: %g\n",headerBuf,totalChar(ch));
    printCchar(ch);
  }
  for(j=0;j<NUMCHAR;j++)
    chs[j] += ch[j];
  return numSeq;
}

/* Count the occurrences of characters in an input sequence. */
int main(int argc, char *argv[]){
  FILE *fp;
  int i, j, bufferSize;
  int numSeq;
  short *dic;
  char *version, *headerBuf;
  Args *args;
  double ch[NUMCHAR], chs[NUMCHAR];

  version = "1.13";
  args = getArgs(argc, argv);
  /* Create dictionary of alphanumerical characters. */
  dic = (short *)emalloc(sizeof(short)*NUMCHAR);
  for(i=0;i<NUMCHAR;i++){
    chs[i] = 0;
    if(isalnum((char)i))
      dic[i] = 1;
    else
      dic[i] = 0;
  }
  bufferSize = args->b;
  headerBuf = (char *)emalloc((bufferSize+1)*sizeof(char));
  numSeq = 0;
  if(args->h || args->e)
    printUsage(version);
  if(args->v)
    printSplash(version);
  if(args->numInputFiles == 0){
    fp = stdin;
    numSeq = scanFile(args, fp, dic, headerBuf, ch);
    printf("# Total number of input characters: %.0f\n", totalChar(ch));
    printCchar(ch);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      if(args->numInputFiles > 1)
	printf("# %s\n",args->inputFiles[i]);
      numSeq += scanFile(args, fp, dic, headerBuf, ch);
      for(j=0;j<NUMCHAR;j++){
	chs[j] += ch[j];
      }
      fclose(fp);
    }
    if(!(args->s && numSeq < 2)){
      printf("# Total number of input characters: %.0f\n", totalChar(chs));
      printCchar(chs);
    }
  }
  free(args);
  free(progname());

  return 0;
}

