/***** stringUtil.c **********************************************
 * Description: Collection of string handling functions.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Jun  6 10:02:16 2004.
 ****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include "stringUtil.h"
#include "eprintf.h"

/* chomp: remove carriage return from string */
char *chomp(char *line){
    char *ol;
    
    ol = line;
    line = strpbrk(line,"\n");
    if(line != NULL)
      *line = '\0';
    return ol;
}

char *strtolower(char *s){
  char *p;
  int i,l;
  l = strlen(s);
  p = (char *) malloc(l+1);
  if(p != NULL){
    for(i=0;i<l;i++)
      p[i] = tolower(s[i]);
    p[i] = '\0';
  }
  return p;  
}

/* printWrap: print and wrap a string */
void printWrap(FILE *fp, char *line, int l){
  int i, c;
  c = 0;
  for(i=0;i<strlen(line);i++){
    if(c<l){
      fprintf(fp,"%c",line[i]);
      c++;
    }else{
      fprintf(fp,"\n%c",line[i]);
      c = 1;
    }
  }
  fprintf(fp,"\n");
}

/* strdup: make a duplicate of s */
char *strdup2(char *s){
  char *p;
  p = (char *) malloc(strlen(s)+1);    /* +1 for '\0' */
  if (p != NULL)
    p = strcpy(p,s);
  return p;
}

void split(char *line, char *splitC, char *splitArray[], int *arrayLen){
  char *line2, c;
  int i, j, start, end;

  start = end = 0;
  *arrayLen = 0;
  line2 = (char *) emalloc(strlen(line)+1);    /* +1 for '\0' */
  line2 = strcpy(line2,line);
  i = 0;
  /* count number of fields */
  while((c = line2[end++]) != '\0'){
    if(c == *splitC){
      splitArray[*arrayLen] = (char *) emalloc((end - start)*sizeof(char));
      i=0;
      for(j=start;j<end-1;j++){
	splitArray[*arrayLen][i++] = line2[j];
      }
      splitArray[*arrayLen][i] = '\0';
      start = end;
      (*arrayLen)++;
      if(*arrayLen > MAXFIELDS){
	printf("ERRROR: cannot deal with more than %d fields in split array.\n",MAXFIELDS);
	assert(0);
      }
	  
    }
  }
  i=0; 
  splitArray[*arrayLen] = (char *) emalloc((end-start+1)*sizeof(char));
  for(j=start;j<end;j++){
    splitArray[*arrayLen][i++] = line2[j];
  }
  splitArray[*arrayLen][i] = '\0';
  (*arrayLen)++;
}

/* remove leading and trailing non-alphanumerical characters */
char *cleanWordEdges(char *word){
  int l1, l2;
  char *word2 = NULL;

  /* remove leading non-alphanumericals */
  while(!isalnum(*word) && *word != '\0'){
    word++;
  }

  /* remove trailing non-alphanumericals */
  l1 = strlen(word);
  l2 = l1;
  while(l2 > 0 && !isalnum(word[l2-1])){
    l2--;
  }
  if(l2 < l1){
    if((word2 = (char *)malloc((l2+1)*sizeof(char))) == NULL){
      printf("ERROR [stringUtil]: memory allocation failed.\n");
      assert(0);
    }
    word2 = strncpy(word2,word,l2);
    return word2; 
  }else{
    return word;
  }
  
}

/* remove all non-alphanumerical characters */
char *cleanWord(char *word){
  int i, j, l1, l2;
  char *word2 = NULL;

  /* get word length */
  l1 = strlen(word);
  
  /* count alphanumericals */
  l2=0;
  for(i=0;i<l1;i++)
    if(isalnum(word[i]))
      l2++;
  if(l2 < l1){  /* create cleaned string */
    if((word2 = (char *)malloc((l2+1)*sizeof(char))) == NULL){
      printf("ERROR [stringUtil]: memory allocation failed.\n");
      assert(0);
    }
    j=0;
    for(i=0;i<l2;i++)
      if(isalnum(word[i]))
	 word2[j++] = word[i];
    word2[j] = '\0';
    return word2; 
  }else{
    return word;
  }
  
}

/* itoa: convert n to characters in s */
void itoa (int n, char *s){
  int i, sign;
  if((sign = n) < 0)       /* record sign */
    n = -n;
  i = 0;
  do{                      /* generate digits in reverse order */
    s[i++] = n % 10 + '0'; /* getnext digit */
  }while ((n /= 10) > 0);
  if(sign < 0)
    s[i++] = '-';
  s[i] = '\0';
  reverse(s);
}

/* reverse: reverse stirng s in place */
void reverse(char *s){
  int c, i, j;
  for(i=0, j=strlen(s)-1; i<j; i++, j--){
    c = s[i];
    s[i] = s[j];
    s[j] = c;
  }
}

/* hash: form hash value for string s */
unsigned hash(char *s){
  unsigned hashval;
  for(hashval = 0; *s != '\0'; s++)
    hashval = *s +31 * hashval;

  return hashval % HASHSIZE;
}

/******************************** Word Counting with Binary Tree *******************************/
WordNode *addWord(WordNode *p, char *w){
  int cond;

  if(p == NULL){                       /* a new word has arrived */
    p = walloc();                      /* make a new node */
    p->word = strdup2(w);
    p->count = 1;
    p->left = p->right = NULL;
  }else if((cond = strcmp(p->word, w)) == 0)
    p->count++;       /* repeated word */
  else if(cond < 0)   /* less than into left subtree */
    p->left = addWord(p->left, w);
  else                /* greater than into right subtree */
    p->right = addWord(p->right, w);
  return p;
}

/* walloc: make WordNode */
WordNode *walloc(void){
  return (WordNode *) malloc(sizeof(WordNode));
}


/* treeprint: in-order print of tree p */
void treeprint(WordNode *p){
  if(p != NULL){
    treeprint(p->left);
    printf("%d %s\n",p->count,p->word);
    treeprint(p->right);
  }
}
/******************************** End of Word Counting with Binary Tree *******************************/
