/***** draw.c *****************************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Fri Sep 23 20:50:06 2005.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "interface.h"
#include "drawGenealogy.h"
#include "eprintf.h"

void draw(Args *args, Indiv ***pop){
  FILE *fp;
  int i, j;
  Gene *gene;

  fp = efopen(args->o,"w");
  fprintf(fp,"\\begin{pspicture}(-1,0)(%d,%d)\n",args->p+2,-args->g);

  for(i=0;i<args->g;i++){
    fprintf(fp,"\\rput(-1,%d){$g_{%d}$} ",-i,i+1);
    fprintf(fp,"\\rput(%d,%d){$b_{%d}$} ",args->p,-i,args->g-i-1); 
    for(j=0;j<args->p;j++){
      if(pop[i][j]->isUa){
	if(args->C)
	  fprintf(fp,"\\psset{linecolor=red}\n");
	else
	  fprintf(fp,"\\psset{doubleline=true,linestyle=solid}\n");
      }else if(pop[i][j]->isNonUa){
	if(args->C)
	  fprintf(fp,"\\psset{linecolor=blue}\n");
	else
	  fprintf(fp,"\\psset{doubleline=false,linestyle=dotted}\n");
      }else{
	if(args->C)
	  fprintf(fp,"\\psset{linecolor=black}\n");
	else
	  fprintf(fp,"\\psset{doubleline=false,linestyle=solid}\n");
      }
      if(pop[i][j]->isMale)
	fprintf(fp,"\\rput(%d,%d){\\psframebox{\\dotnode{%d%d1}\\, \\, \\dotnode{%d%d2}}}",j,-i,j,i,j,i);
      else
	fprintf(fp,"\\rput(%d,%d){\\psovalbox{\\dotnode{%d%d1}\\, \\, \\dotnode{%d%d2}}}",j,-i,j,i,j,i);
    }
    fprintf(fp,"\n");
  }
  for(i=0;i<args->p;i++)
    fprintf(fp,"\\rput(%d,%d){$i_{%d}$}",i,-args->g,i+1);
  fprintf(fp,"\n");
  fprintf(fp,"\\rput(%d,%d){\\Rnode{Present}{\\psshadowbox{\\scalebox{1.0}{Present}}}}\n",args->p+1,-args->g+1);
  fprintf(fp,"\\rput(%d,%d){\\Rnode{Past}{\\psshadowbox{\\scalebox{1.0}{Past}}}}\n",args->p+1,0);
  fprintf(fp,"\\ncline{->,linewidth=2pt}{Present}{Past}\n");
  fprintf(fp,"\\psset{doubleline=false,linestyle=solid}\n");

  if(args->i){
    if(args->C)
      fprintf(fp,"\\psset{linecolor=green}");
    else
      fprintf(fp,"\\psset{linecolor=blue}");
    followIndividuals(fp,pop[args->g-1][args->i-1],args);
    if(args->C)
      fprintf(fp,"\\psset{linecolor=magenta}");
    else
      fprintf(fp,"\\psset{linecolor=red}");
    followGenes(fp,pop[args->g-1][args->i-1]->gene1,args);
    followGenes(fp,pop[args->g-1][args->i-1]->gene2,args);
  }else{
    fprintf(fp,"\\psset{linecolor=lightgray}");
    for(i=args->g-1;i>0 && i>args->d;i--){
      for(j=0;j<args->p;j++){
	gene = pop[i][j]->gene1;
	if(gene->id)
	  fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,1);
	else
	  fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,2);
	if(gene->ancestor->id)
	  fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,1);
	else
	  fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,2);
	fprintf(fp,"\n");
	gene = pop[i][j]->gene2;
	if(gene->id)
	  fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,1);
	else
	  fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,2);
	if(gene->ancestor->id)
	  fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,1);
	else
	  fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,2);
	fprintf(fp,"\n");
      }
    }
  }
  fprintf(fp,"\n\\end{pspicture}\n");
  fclose(fp);
}

void followGenes(FILE *fp, Gene *gene, Args *args){

  if(gene->ancestor != NULL && gene->generation > args->D){
    if(gene->id)
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,1);
    else
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,2);
    if(gene->ancestor->id)
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,1);
    else
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,2);
    fprintf(fp,"\n");
    followGenes(fp,gene->ancestor,args);
  }

}

void followIndividuals(FILE *fp, Indiv *indiv, Args *args){

  Gene *gene;
  
  if(indiv->ancestor1 != NULL){
    if(indiv->generation > args->d)
      followIndividuals(fp,indiv->ancestor1,args);
    
    gene = indiv->gene1;
    if(gene->id)
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,1);
    else
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,2);
    if(gene->ancestor->id)
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,1);
    else
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,2);
    fprintf(fp,"\n");
    gene = indiv->gene2;
    if(gene->id)
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,1);
    else
      fprintf(fp,"\\ncline{%d%d%d}",gene->pos,gene->generation,2);
    if(gene->ancestor->id)
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,1);
    else
      fprintf(fp,"{%d%d%d}",gene->ancestor->pos,gene->ancestor->generation,2);
    fprintf(fp,"\n");
    if(indiv->generation > args->d)
      followIndividuals(fp,indiv->ancestor2,args);
  }

}

void printLatexHeader(FILE *fp){
  fprintf(fp,"\\documentclass{article}\n");
  fprintf(fp,"\\usepackage{pstricks,pst-node,graphics,times}\n");
  fprintf(fp,"\\oddsidemargin=0cm\n");
  fprintf(fp,"\\evensidemargin=0cm\n");
  fprintf(fp,"\\pagestyle{empty}\n");
  fprintf(fp,"\\textwidth=16cm\n");
  fprintf(fp,"\\textheight=23cm\n\n");
  fprintf(fp,"\\begin{document}\n");
  fprintf(fp,"\\title{Random Family Histories}\\author{}\\date{}\\maketitle\n");
}
