program draw_strees
  !
  ! Purpopse: 
  !   Draw a generalized suffix tree.
  !   Reference: E. Ukkonen (1995). On-line construction of suffix trees. Algorithmica
  !              14, 249-260.
  ! Copyright (C) 2004 Bernhard Haubold
  ! Software distributed under the terms of the GNU GENERAL PUBLIC LICENSE
  ! Record of revisions:
  !    Date            Programmer           Description of change
  !    ====            ==========           =====================
  !  May 02, 2004      B. Haubold           Original code assembled from shustring code basis
  !
  use user_interface
  use stree
  use sequence_input
  implicit none
  character(256)                               :: file_name
  character(256), pointer, dimension(:)        :: argv              ! Argument vector
  character(256), pointer,dimension(:)         :: names             ! Names of sequences
  type(text_row), pointer, dimension(:)        :: combined_text     ! Input sequences
  integer   , parameter                        :: max_seq=10**5     ! Maximum number of sequences accepted
  integer   , parameter                        :: max_line=200      ! Max. length of lines in FASTA file
  logical                                      :: fdna=.true.       ! Whether or not to filter out ambiguity from DNA
  type(arguments)                              :: args              ! The command line arguments
  type(node), pointer                          :: root              ! Root of suffix tree
  integer, parameter                           :: molecule_type = 5 ! Treat input sequences as single stranded
  integer                                      :: ios, iargc, stat, u, text_size, status
  logical                                      :: open 
  character(4), parameter                      :: version='1.3'
  character(256)                               :: psfigure='psfigure'
  
  interface
     recursive subroutine draw_tree(root, text, file, circle_node)  
       use sequence_input
       use stree
       implicit none
       type(node), pointer                   :: root
       type(text_row), dimension(:), pointer :: text        ! Text of the suffix tree
       character(256), intent(in)            :: file        ! Name of PsTricks file
       logical, intent(in)                   :: circle_node ! Defines the node type !circle means cnode
     end subroutine draw_tree
  end interface

  ! Get data`
  call get_argv(argv)
  call construct_arguments(args)
  stat=1
  call get_arguments(argv, args, stat)
  if(stat.lt.0) goto 100
  if(args%h) then
     call argument_list(version)
     goto 100
  end if
  if(args%a) then
     call splash(version)
     goto 100
  end if
  if(args%i.eq.'') then             
     call argument_list(version)
     goto 100
  else if(args%i.eq.'stdin') then
     u = 5
  else
     ! Open the specified input file
     u = 2
     open(unit=u, iostat=ios,  file=args%i)
     if(ios.gt.0) then
        write(6,'(2a)') '[drawStrings] ERROR: Invalid input file ', trim(args%i)
        goto 100
     end if
  end if
  call read_fasta(u, combined_text, names, max_seq, max_line)
  close(u)
  call prepare_data(combined_text,molecule_type,args%s)
  call construct_stree(combined_text, root)
  if(args%n.eq.'circle')then
     call draw_tree(root, combined_text, args%o, .true.)
  else
     call draw_tree(root, combined_text, args%o, .false.)
  end if

100 end program draw_strees

