module string_utils
  !
  ! Purpopse: 
  !   Collection of string manipulation routines
  ! Copyright (C) 2004 Bernhard Haubold
  ! Software distributed under the terms of the GNU GENERAL PUBLIC LICENSE
  ! Record of revisions:
  !    Date            Programmer           Description of change
  !    ====            ==========           =====================
  !  May 02, 2004      B. Haubold           Original code
  !
  interface char_sar
     module procedure char_sar1, char_sar2
  end interface

  interface tra_uc
     module procedure tra_uc1, tra_uc2
  end interface

  interface reverse
     module procedure reverse1, reverse2
  end interface

  interface complement
     module procedure complement1, complement2
  end interface

  interface reverse_complement
     module procedure reverse_complement1, reverse_complement2
  end interface
  
contains
! Character search and replace in two versions
  subroutine char_sar1(co, cn, str)
        implicit none
    character(1), intent(in)     :: co, cn
    character(*)                 :: str
    integer    i, str_len

    str_len = len(str)
    do i=1,str_len
       if(str(i:i).eq.co) str(i:i)=cn
    end do

  end subroutine char_sar1
  
  subroutine char_sar2(co, cn, str)
        implicit none
    character(1), intent(in)                    :: co, cn
    character(1), dimension(:), intent(inout)   :: str
    integer    i, str_len

    str_len = size(str)
    do i=1,str_len
       if(str(i).eq.co) str(i)=cn
    end do
  
  end subroutine char_sar2

 
  ! Translate to upper case in three versions
  subroutine tra_uc1(str)
        implicit none
    character(*)                  :: str
    integer    i, isize, iasc

    isize = len(str)
    do i=1,isize
       iasc = ichar(str(i:i))
       if((iasc.gt.96).and.(iasc.LT.123))then
          str(i:i) = char(iasc-32)
       end if
    end do
  end subroutine tra_uc1
  
  subroutine tra_uc2(str)
        implicit none
    character(1), pointer, dimension(:)         :: str
    integer    i, isize, iasc

    isize = size(str)
    do i=1, isize
       iasc = ichar(str(i))
       if((iasc.gt.96).and.(iasc.LT.123))then
          str(i) = char(iasc-32)
       end if
    end do
  end subroutine tra_uc2

  ! Reverse string in two versions
  subroutine reverse1(str)
        implicit none
    character(*)                            :: str
    character(1), dimension(:), allocatable :: str2
    integer    i, j, isize
    
    isize=len(str)
    allocate(str2(isize))
    j=1
    do i=isize,1,-1
       str2(j)=str(i:i)
       j=j+1
    end do
    do i=1,isize
       str(i:i)=str2(i)
    end do
    deallocate(str2)
  end subroutine reverse1

  subroutine reverse2(str)
        implicit none
    character(1), dimension(:)              :: str
    character(1), dimension(:), allocatable :: str2
    integer    i, j, isize
    
    isize=size(str)
    allocate(str2(isize))
    j=1
    do i=isize,1,-1
       str2(j)=str(i)
       j=j+1
    end do
    str(:)=str2(:)
    deallocate(str2)
  end subroutine reverse2

  ! Complement string in two versions
  subroutine complement1(str)
        implicit none
    character(*)                            :: str
    integer    i, isize
    
    isize=len(str)
    do i=1,isize
       select case (str(i:i))
       case('A')
          str(i:i)='T'
       case('C')
          str(i:i)='G'
       case('G')
          str(i:i)='C'
       case('T')
          str(i:i)='A'
       case('U')
          str(i:i)='U'
       case default
          write(6,'(a)') 'ERROR: string_utils - trying to complement ',str(i:i)
          stop
       end select
    end do
  end subroutine complement1
  
  subroutine complement2(str)
        implicit none
    character(1), dimension(:)              :: str
    integer    i,isize

    isize=size(str)
    do i=1,isize
       select case (str(i))
       case('A')
          str(i)='T'
       case('C')
          str(i)='G'
       case('G')
          str(i)='C'
       case('T')
          str(i)='A'
       case('U')
          str(i)='U'
       case default
          write(6,'(a)') 'ERROR: string_utils - trying to complement ',str(i)
          stop
       end select
    end do
  end subroutine complement2

  ! Reverse and complement a DNA molecule in two versions
  subroutine reverse_complement1(str)
    implicit none
    character(*)                            :: str
    
    call reverse(str)
    call complement(str)
  end subroutine reverse_complement1

  subroutine reverse_complement2(str)
        implicit none
    character(1), dimension(:), intent(inout)  :: str
    
    call reverse(str)
    call complement(str)
  end subroutine reverse_complement2

end module string_utils

