module user_interface
  !
  ! Purpopse: 
  !    Provide user interface for the program drawStrees
  ! Copyright (C) 2004 Bernhard Haubold
  ! Software distributed under the terms of the GNU GENERAL PUBLIC LICENSE
  ! Record of revisions:
  !    Date            Programmer           Description of change
  !    ====            ==========           =====================
  !  Mar. 20, 2004     B. Haubold           Original code
  !
  type arguments
     character(256)      :: i  ! Name of input sequence
     character(256)      :: o  ! Name of output file
     character           :: s  ! Sentinel character
     character(20)       :: n  ! Node type
     logical             :: h  ! Whether or not to print help
     logical             :: a  ! Whether or not to print version information
  end type arguments
contains

  subroutine get_argv(argv)
    !
    ! Purpopse: 
    !    Get the command line arguments as entries in the array argv.
    ! Record of revisions:
    !    Date            Programmer           Description of change
    !    ====            ==========           =====================
    !  Jan. 14, 2004     B. Haubold           Original code
    !
    implicit none
    character(256), pointer, dimension(:) :: argv
    integer                               :: iargc
    integer    i

    allocate(argv(iargc()))

    do i=1,size(argv)
       call getarg(i,argv(i))
    end do
  end subroutine get_argv

  subroutine construct_arguments(args)
    !
    ! Purpopse: 
    !    Construct the argument transport object with initial values
    ! Record of revisions:
    !    Date            Programmer           Description of change
    !    ====            ==========           =====================
    !  Jan. 14, 2004     B. Haubold           Original code
    !
    implicit none
    type(arguments), intent(out)          :: args  ! The argument transport object

    args%i=''
    args%o='pstricks_file'
    args%s='$'
    args%n='dot'
    args%a=.false.
    args%h=.false.
  end subroutine construct_arguments

  subroutine get_arguments(argv, args, stat)
    !
    ! Purpopse: 
    !    Get the command line arguments in argv as entries in the transport object args
    ! Record of revisions:
    !    Date            Programmer           Description of change
    !    ====            ==========           =====================
    !  Jan. 14, 2004     B. Haubold           Original code
    !
    use string_utils
    implicit none
    character(256), pointer, dimension(:) :: argv  ! The argument vector
    type(arguments), intent(out)          :: args  ! The argument transport object
    integer   , intent(out)               :: stat  ! Status; set < 0 for error
    integer                               :: i
    character(1)                          :: chr, c
    integer                               :: flag

    i=1

    do
       if(i.gt.size(argv)) exit
       c=''
       call char_sar('-',c, argv(i)(1:len_trim(argv(i))))
       chr = argv(i)(2:2)
       select case(chr)
       case('i')
          args%i = argv(i+1)
          i = i+2
       case('o')
          args%o = argv(i+1)
          i = i+2
       case('s')
          args%s = argv(i+1)
          if(args%s.eq.'%') args%s=''
          i = i+2
       case('n')
          args%n = argv(i+1)
          i = i+2
       case('a')
          args%a = .true.
          i = i+1
       case('h')
          args%h = .true.
          i = i+1
       case default
          write(6,'(3a)') '[fourall] ERROR: Invalid argument: ', '-',trim(argv(1))
          stat=-1
          exit
       end select
    end do
  end subroutine get_arguments

  subroutine argument_list(version)
    !
    ! Purpopse: 
    !    Print out the argument list
    ! Record of revisions:
    !    Date            Programmer           Description of change
    !    ====            ==========           =====================
    !  Jan. 14, 2004     B. Haubold           Original code
    !
    implicit none
    character(*), intent(in)          :: version  ! The current version of the program

    write(6,'(3a)') 'drawStrees version ', trim(version) , ', written by Bernhard Haubold'
    write(6,'(a)')  'purpose: draw generalized suffix trees in LaTeX format'
    write(6,'(a)')  'usage: drawStrees [options]'
    write(6,'(a)') 'options: '
    write(6,*) '-i  input file in FASTA format'
    write(6,*) '-o  output file'
    write(6,*) '-s  sentinel character; enter % for adding no sentinel', char(10), char(9),   & 
         'default: $'
    write(6,*) '-n  node type [dot|circle]', char(10), char(9),   & 
         'default: dot'
    write(6,*) '-h  print option list'
    write(6,*) '-a  print out author and licence information'

  end subroutine argument_list

  subroutine splash(version)
    !
    ! Purpopse: 
    !    Provide information about a program
    ! Record of revisions:
    !    Date            Programmer           Description of change
    !    ====            ==========           =====================
    !  Jan. 10, 2004     B. Haubold           Original code
    !
    implicit none
    character(*), intent(in)  :: version

    write(6,'(a)') '***************************************************'
    write(6,'(3a)')'*                 drawStrees ', version, '                 *'
    write(6,'(a)') '*  Program for drawing generalized suffix trees   *'
    write(6,'(a)') '*                        by                       *'  
    write(6,'(a)') '*                Bernhard Haubold                 *'
    write(6,'(a)') '*-------------------------------------------------*'
    write(6,'(a)') '* Max-Planck-Institute for Evolutionary Biology   *'
    write(6,'(a)') '* Department of Evolutionary Genetics             *'
    write(6,'(a)') '* haubold@evolbio.mpg.de                          *'
    write(6,'(a)') '*-------------------------------------------------*'
    write(6,'(a)') '* NOTES:                                          *'
    write(6,'(a)') '* 1) To find out more about suffix trees          *'
    write(6,'(a)') '*    consult: Gusfield, D. (1997). Algorithms     *'
    write(6,'(a)') '*    on Strings, Trees, and Sequences: Computer   *'
    write(6,'(a)') '*    Science and Computational Biology. CUP.      *'
    write(6,'(a)') '* 2) The latest version of this program is        *'
    write(6,'(a)') '*    available at                                 *'
    write(6,'(a)') '*    http://guanine.evolbio.mpg.de/drawStrees/    *'
    write(6,'(a)') '* LICENCE:                                        *'
    write(6,'(a)') '* drawStree is free software. It is distributed   *'
    write(6,'(a)') '* under the terms of the GNU General Public       *'
    write(6,'(a)') '* License. A copy of this licence can be obtained *'
    write(6,'(a)') '* from www.gnu.org                                *'
    write(6,'(a)') '***************************************************'
  end subroutine splash
end module user_interface
