/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Jun 20 13:12:10 2004.
 *
 * This file is part of drawWrightFisher.
 *
 *   drawWrightFisher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   drawWrightFisher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with drawWrightFisher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "interface.h"
#include "eprintf.h"

Args *getArgs(int argc, char *argv[]){
  Args *args;
  int c;
  int arg; 

  args = (Args *)emalloc(sizeof(Args));

  args->p = -1;         /* population size */
  args->n = -1;         /* sample size */
  args->o = "wrightFisher.tex";       /* output file */
  args->t = NULL;
  args->g = 10;

  args->s = 0;
  args->a = NULL;
  args->m = 0;
  args->f = 0;
  args->h = 0;
  args->e = 1;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'n':
	args->n = (int)atoi(argv[++arg]);   /* sample size */
	break;
      case 'p':
	args->p = (int)atoi(argv[++arg]);   /* population size */
	args->e = 0;
	break;
      case 'g':                            /* generations */
	args->g = (int)atoi(argv[++arg]);  
	break;
     case 'o':                            /* output file */
	args->o = argv[++arg];  
	break;
      case 'a':                           /* ancestral lineages? */
	args->a = argv[++arg];  
	break;
      case 'm':
	args->m = 1;                      /* show MRCA? */
	break;
      case 'f':                           /* mark forward? */
	args->f = (int)atoi(argv[++arg]);  
	break;
     case 't':                            /* test file */
	args->t = argv[++arg];  
	break;
      case 's':                            /* seed for random number generator */
	args->s = (int)atoi(argv[++arg]);
	if(args->s < 0)
	  args->s = -args->s;
	break;
      case 'h':                            /* print help */
	args->h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args->e = 1;
	return args;
      }
      arg++;
  }

  return args;
}

void printUsage(char *version){
  printf("drawWrightFisher version %s, copyright (c) 2006 Bernhard Haubold\n", version);
  printf("\tdistributed under the GNU General Public License\n");
  printf("purpose: draw lineages in Wright-Fisher population\n");
  printf("arguments:  \n");
  printf("\t-p NUM population size=NUM\n");
  /*printf("\t-n sample size\n"); */
  printf("\t[-g NUM run for NUM generations; default: NUM=10]\n");
  printf("\t[-s NUM use NUM as seed for random number generator; default: system clock]\n");
  printf("\t[-o FILE write output to FILE; default: FILE wrightFisher.tex]\n");
  printf("\t[-t FILE write test file to FILE; default: no test file]\n");
  printf("\t[-a NUMLIST mark ancestral lineages for genes in NUMLIST, e.g. 1,3; -1 for all;\n");
  printf("\t\tdefault: all lineages identical]\n");				       
  printf("\t[-m mark most recent common ancestor; default: draw all genes in same style]\n");
  printf("\t[-f NUM forward from gene NUM in last generation; default: draw all genes in same style]\n");				    
  printf("\t[-h print this help message]\n");						    
												
}
