/***** code.c *************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Nov  1 18:24:58 2016
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "eprintf.h"
#include "aa.h"
#include "code.h"
#include "prop.h"
#include "gsl_rng.h"

int ***getTri(){
  int i, j;
  int ***tri;

  /* allocate memory */
  tri = (int ***)emalloc(4*sizeof(int **));
  for(i=0;i<4;i++){
    tri[i] = (int **)emalloc(4*sizeof(int *));
    for(j=0;j<4;j++)
      tri[i][j] = (int *)emalloc(4*sizeof(int));
  }
  /* assign amino acids */
  tri[0][0][0] = tri[0][0][1] = 0;  /* Phe */
  tri[0][0][2] = tri[0][0][3] = 1;  /* Leu */
  for(i=0;i<4;i++)
    tri[0][1][i] = 2;                /* Ser */
  tri[0][2][0] = tri[0][2][1] = 3;  /* Tyr */
  tri[0][2][2] = tri[0][2][3] = 20; /* Ter */
  tri[0][3][0] = tri[0][3][1] = 4;  /* Cys */
  tri[0][3][2] = 20;                 /* Ter */
  tri[0][3][3] = 5;                  /* Trp */

  for(i=0;i<4;i++){
    tri[1][0][i] = 1;                /* Leu */
    tri[1][1][i] = 6;                /* Pro */
  }
  tri[1][2][0] = tri[1][2][1] = 7;  /* His */
  tri[1][2][2] = tri[1][2][3] = 8;  /* Gln */
  for(i=0;i<4;i++)
    tri[1][3][i] = 9;                /* Arg */

  for(i=0;i<3;i++)
    tri[2][0][i] = 10;               /* Ile */
  tri[2][0][3] = 11;                 /* Met */
  for(i=0;i<4;i++)
    tri[2][1][i] = 12;               /* Thr */
  tri[2][2][0] = tri[2][2][1] = 13; /* Asn */
  tri[2][2][2] = tri[2][2][3] = 14; /* Lys */
  tri[2][3][0] = tri[2][3][1] = 2;  /* Ser */
  tri[2][3][2] = tri[2][3][3] = 9;  /* Arg */

  for(i=0;i<4;i++){
    tri[3][0][i] = 15;               /* Val */
    tri[3][1][i] = 16;               /* Ala */
  }
  tri[3][2][0] = tri[3][2][1] = 17; /* Asp */
  tri[3][2][2] = tri[3][2][3] = 18; /* Glu */
  for(i=0;i<4;i++)
    tri[3][3][i] = 19;               /* Gly */
  
  return tri;
}

int *getMap(){
  int *map, i;

  map = (int *)emalloc(21*sizeof(int));
  for(i=0;i<21;i++)
    map[i] = i;
  
  return map;
}

char *codonToAa1(int codon, Code *code){
  int ind = code->map[codon];
  return code->aa1[ind];
}

char *codonToAa3(int codon, Code *code){
  int ind = code->map[codon];
  return code->aa3[ind];
}

double codonToProp(int codon, Code *code){
  int ind = code->map[codon];
  return code->pro[ind];
}

void findNsPairs1(int i, int j, int k, int l, Code *c){
  int id1, id2;
  id1 = c->tri[i][j][k];
  id2 = c->tri[l][j][k];
  if(id1 != 20 &&  id2 != 20){ /* not Ter */
    c->fn1++;
    if(id1 != id2){
      c->n1++;
      c->pairs1 = (int **)erealloc(c->pairs1,c->n1*sizeof(int *));
      c->pairs1[c->n1-1] = (int *)emalloc(2*sizeof(int));
      c->pairs1[c->n1-1][0] = id1;
      c->pairs1[c->n1-1][1] = id2;
    }
  }
}

void findNsPairs2(int i, int j, int k, int l, Code *c){
  int id1, id2;
  id1 = c->tri[i][j][k];
  id2 = c->tri[i][l][k];
  if(id1 != 20 &&  id2 != 20){ /* not Ter */
    c->fn2++;
    if(id1 != id2){
      c->n2++;
      c->pairs2 = (int **)erealloc(c->pairs2,c->n2*sizeof(int *));
      c->pairs2[c->n2-1] = (int *)emalloc(2*sizeof(int));
      c->pairs2[c->n2-1][0] = id1;
      c->pairs2[c->n2-1][1] = id2;
    }
  }
}

void findNsPairs3(int i, int j, int k, int l, Code *c){
  int id1, id2;
  id1 = c->tri[i][j][k];
  id2 = c->tri[i][j][l];
  if(id1 != 20 &&  id2 != 20){ /* not Ter */
    c->fn3++;
    if(id1 != id2){
      c->n3++;
      c->pairs3 = (int **)erealloc(c->pairs3,c->n3*sizeof(int *));
      c->pairs3[c->n3-1] = (int *)emalloc(2*sizeof(int));
      c->pairs3[c->n3-1][0] = id1;
      c->pairs3[c->n3-1][1] = id2;
    }
  }
}

void getNormalizationFactors(Code *c){
  int i, j, k, l;

  c->fn1 = c->fn2 = c->fn3 = 0;
  c->n1 = c->n2 = c->n3 = 0;
  c->pairs1 = c->pairs2 = c->pairs3 = NULL;
  for(i=0;i<4;i++)
    for(j=0;j<4;j++)
      for(k=0;k<4;k++)
	for(l=0;l<4;l++){
	  if(l != k) /* looking at third codon position */
	    findNsPairs3(i, j, k, l, c);
	  if(l != j) /* looking at second codon position */
	    findNsPairs2(i, j, k, l, c);
	  if(l != i) /* looking at first codon position */
	    findNsPairs1(i, j, k, l, c);
	}
}

Code *getCode(FILE *fp){
  Code *code;

  code = (Code *)emalloc(sizeof(Code));
  code->tri = getTri();
  code->map = getMap();
  code->aa3 = getAa3();
  code->aa1 = getAa1();
  code->nuc = "TCAG";
  getProp(fp,code);
  getNormalizationFactors(code);

  return code;
}

void printCode(Code *c){
  int i, j, k;

  for(i=0;i<4;i++)
    printf("   %c",c->nuc[i]);
  printf("\n");

  for(i=0;i<4;i++){
    printf("%c",c->nuc[i]);
    for(j=0;j<4;j++){
      if(j)
	printf(" ");
      for(k=0;k<4;k++)
	printf(" %s",codonToAa3(c->tri[i][k][j],c));
      printf(" %c\n",c->nuc[j]);
    }
  }
}

void analyzeCode(Code *c){
  int i;
  double p1, p2, d;

  c->ms1 = c->ms2 = c->ms3 = 0.;
  /* first codon position */
  for(i=0; i < c->n1; i++){
    p1 = c->pro[c->map[c->pairs1[i][0]]];
    p2 = c->pro[c->map[c->pairs1[i][1]]];
    d = p1 - p2;
    c->ms1 += d * d;
  }
  /* second codon position */
  for(i=0; i < c->n2; i++){
    p1 = c->pro[c->map[c->pairs2[i][0]]];
    p2 = c->pro[c->map[c->pairs2[i][1]]];
    d = p1 - p2;
    c->ms2 += d * d;
  }
  /* third codon position */
  for(i=0; i < c->n3; i++){
    p1 = c->pro[c->map[c->pairs3[i][0]]];
    p2 = c->pro[c->map[c->pairs3[i][1]]];
    d = p1 - p2;
    c->ms3 += d * d;
  }
  c->ms0 = (c->ms1 + c->ms2 + c->ms3) / (c->fn1 + c->fn2 + c->fn3);
  c->ms1 /= c->fn1;
  c->ms2 /= c->fn2;
  c->ms3 /= c->fn3;
}

void shuffleMap(Code *c, gsl_rng *ran){
  int i, r, tmp;

  for(i=19;i>0;i--){
    r = gsl_rng_uniform(ran) * 20;
    tmp = c->map[r];
    c->map[r] = c->map[i];
    c->map[i] = tmp;
  }
}

void freeCode(Code *c){
  int i, j;
  
  for(i=0;i<4;i++){
    for(j=0;j<4;j++){
      free(c->tri[i][j]);
    }
    free(c->tri[i]);
  }
  free(c->tri);
  free(c->map);
  for(i=0;i<20;i++){
    free(c->aa1);
    free(c->aa3);
  }
  free(c->aa1);
  free(c->aa3);
  free(c->pro);
  free(c);
}
