/***** genCode.c **********************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Nov  1 17:40:48 2016
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "interface.h"
#include "eprintf.h"
#include "code.h"
#include "prop.h"

void runAnalysis(FILE *fp, Args *args, gsl_rng *ran){
  int i;
  Code *code;
  double ms0, ms1, ms2, ms3;
  double sms0, sms1, sms2, sms3;

  code = getCode(fp);
  if(args->d){
    printCode(code);
    printProp(code);
    printf("fn1: %d; fn2: %d; fn3: %d\n",code->fn1,code->fn2,code->fn3);
    printf("n1: %d; n2: %d; n3: %d\n",code->n1, code->n2, code->n3);
    return;
  }
  analyzeCode(code);
  ms0 = code->ms0;
  ms1 = code->ms1;
  ms2 = code->ms2;
  ms3 = code->ms3;
  sms0 = sms1 = sms2 = sms3 = 0.;
  for(i=0;i<args->n;i++){
    shuffleMap(code, ran);
    analyzeCode(code);
    sms0 += code->ms0;
    sms1 += code->ms1;
    sms2 += code->ms2;
    sms3 += code->ms3;
    if(args->p)
      printf("ms0:\t%g\n",code->ms0);
    else if(!args->q && code->ms0 < ms0){
      printCode(code);
      printf("ms0: %g\n",code->ms0);
    }
  }
  sms0 /= args->n;
  sms1 /= args->n;
  sms2 /= args->n;
  sms3 /= args->n;
  if(args->m){
    printf("\tms1\tms2\tms3\tms0\n");
    printf("nc\t%.2f\t%.2f\t%.2f\t%.2f\n",ms1,ms2,ms3,ms0);
    printf("m(rc)\t%.2f\t%.2f\t%.2f\t%.2f\n",sms1,sms2,sms3,sms0);
  }else{
    printf("\tms0\n");
    printf("nc\t%.2f\n",ms0);
    printf("m(rc)\t%.2f\n",sms0);
  }
}

int main(int argc, char *argv[]){
  int i;
  char *version;
  Args *args;
  FILE *fp;
  gsl_rng *ran;

  version = "0.4";
  setprogname2("genCode");
  args = getArgs(argc, argv);
  ran = ini_gsl_rng(args);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  if(args->numInputFiles == 0){
    fp = stdin;
    runAnalysis(fp, args, ran);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      runAnalysis(fp, args, ran);
      fclose(fp);
    }
  }
  free_gsl_rng(ran, args);
  free(args);
  free(progname());
  return 0;
}

