/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "hvb:n:,t:,s,S:c";

  args = (Args *)emalloc(sizeof(Args));
  args->S = 0;
  args->s = 0;
  args->c = 0;
  args->t = DEFAULT_T;
  args->n = DEFAULT_N;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'n':                           /* sample size */
      args->n = atoi(optarg);
      break;
    case 't':                           /* theta */
      args->t = atof(optarg);
      break;
    case 's':
      args->s = 1;
      break;
    case 'c':
      args->c = 1;
      break;
    case 'S':
      args->S = atoi(optarg);
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("Generate random phylogenetic tree\n");
  printf("Example: genTree\n");
  printf("Options:\n");
  printf("\t[-n <NUM> number of taxa; default: %d]\n", DEFAULT_N);
  printf("\t[-t <NUM> population mutation rate (\\theta = 4N\\mu); default: %.1f]\n", DEFAULT_T);
  printf("\t[-s branch lengths proportional to speciation times; default: prop. to mutations]\n");
  printf("\t[-c coalescent tree; default: phylogeny]\n");
  printf("\t[-S <NUM> seed for random number generator; default: file, time]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
