/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "stringUtil.h"
#include "histogram.h"
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "LhvR:b:Br";
  char **startEnd;
  int i;

  args = (Args *)emalloc(sizeof(Args));
  args->b = NUMBINS;
  args->R = NULL;
  args->B = 0;
  args->L = 0;
  args->r = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'b':                           /* number of bins */
      args->b = atoi(optarg);
      break;
    case 'R':                           /* range */
      startEnd = (char **)emalloc(2*sizeof(char *));
      split(optarg,",",startEnd,&i);
      if(i<2){
	printf("ERROR[histogram]: malformatted argument to -R %s; use, for example, -R 100,200.\n",optarg);
	return NULL;
      }
      args->R = (double *)emalloc(2*sizeof(double));
      args->R[0] = atof(startEnd[0]);
      args->R[1] = atof(startEnd[1]);
      break;
    case 'B':                           /* box plot */
      args->B = 1;
      break;
    case 'L':                           /* line plot */
      args->L = 1;
      break;
    case 'r':                           /* raw data */
      args->r = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  return args;
}

void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("generate histogram from list of numbers\n");
  printf("Example: histogram numbers.txt\n");
  printf("Options:\n");
  printf("\t[-b <NUM> use NUM bins; default: %d]\n",NUMBINS);			
  printf("\t[-R <NUM1>,<NUM2> range (NUM1, NUM2); default: min,max]\n");
  printf("\t[-B draw box plot; default: step plot]\n");
  printf("\t[-L draw line plot; default: step plot]\n");
  printf("\t[-r print raw counts; default: print fraction]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}

void freeArgs(){
  if(args->R)
    free(args->R);
  free(args);
}
