/***** ktree.c ************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sat Apr 15 12:57:38 2017
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>
#include "ktree.h"
#include "eprintf.h"
#include "stringUtil.h"
#include "construct.h"

/*
 * Get a list of patterns delineated by | and return a pointer
 * to corresponding two-dimensional array of patterns.
 */
char **processPatterns(char *patterns, int *patternSize, int *residues){
  char **p;
  int i;

  if((p = (char **)malloc(MAX_PATTERNS*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory allocation failed\n");
    assert(0);
  }
  i = 0;
  *residues = 0;
  p[i++] = strtok(patterns, "|");
   *residues += strlen(p[i-1]);
  while((p[i++] = strtok(NULL, "|")) != NULL){
    *residues += strlen(p[i-1]);
    ;
  }
  *patternSize = i-1;

  return p;
}

KTree *getKeywordTree(Args *args){
  Sequence *pattern;   /* pattern sequences */
  char **p;            /* pattern data */
  int patternSize, residues;
  KNode *root;
  KTree *ktree;
  int i, j;

  pattern = NULL;
  p = NULL;
  /* get pattern data either from file or from command line*/
  if(args->f != NULL){
    pattern = readFasta(args->f, pattern, &patternSize, &residues);
  }else{
    p = processPatterns(args->p, &patternSize, &residues);
    /* transfer the patterns to a sequence array */
    pattern = (Sequence *)emalloc(patternSize*sizeof(Sequence));
    for(i=0;i<patternSize;i++){
      pattern[i].seq = p[i];
      pattern[i].id = (char *)emalloc((strlen(p[i]) + 1)*sizeof(char));
      pattern[i].id = strcpy(pattern[i].id,p[i]);
    }
  }

  if(args->i){
    for(i=0;i<patternSize;i++){
      for(j=0;pattern[i].seq[j]!='\0';j++){
	pattern[i].seq[j] = tolower(pattern[i].seq[j]);
      }
      pattern[i].seq[j] = '\0';
    }
  }
  ktree = (KTree *)emalloc(sizeof(KTree));
  ktree->pattern = pattern;
  /* Construct keyword tree */
  if((root = createSequenceKeywordTree(pattern, patternSize, residues)) == NULL){
    printf("ERROR [keywordMatcher]: memory allocation failed\n");
    assert(0);
  }
  ktree->root = root;
  
  return ktree;
}
