/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "hvnlw:o:rud:as:";

  args = (Args *)emalloc(sizeof(Args));
  args->d = DEFAULT_D;
  args->s = -1.;
  args->a = 0;
  args->r = 0;
  args->l = 0;
  args->u = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;
  args->n = 0;
  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'n':                            /* no labels? */
      args->n = 1;
      break;
    case 'd':                            /* smallest dimension */
      args->d = atof(optarg);
      break;
    case 's':                            /* scale */
      args->s = atof(optarg);
      break;
    case 'r':                            /* rooted tree */
      args->r = 1;
      break;
    case 'a':                            /* latex? */
      args->a = 1;
      break;
    case 'l':                            /* print id-labels */
      args->l = 1;
      break;
    case 'u':                            /* unrooted tree */
      args->u = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  if(args->u && args->r){
    printf("ERROR: please choose between rooted and unrooted layout.\n");
    args->e = 1;
  }
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("View trees in Newick format\n");
  printf("new2view my.tree\n");
  printf("Options:\n");
  printf("\t[-d NUM smallest dimension (in cm); default: %.1f]\n",DEFAULT_D);
  printf("\t[-s NUM scale; default: find automatically]\n");
  printf("\t[-u draw unrooted tree; default: let input decide]\n");
  printf("\t[-r draw rooted tree; default: let input decide]\n");
  printf("\t[-n no labels; default: print labels]\n");
  printf("\t[-l id labels; default: print text labels]\n");
  printf("\t[-a print LaTeX output; default: generate image]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
