/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  char c;
  char *optString = "hvptm:n:";

  args = (Args *)emalloc(sizeof(Args));
  args->h = 0;
  args->v = 0;
  args->e = 0;
  args->p = 0;
  args->t = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'm':                           /* length of first sequence */
      args->m = atoi(optarg);
      break;
    case 'n':                           /* length of second sequence */
      args->n = atoi(optarg);
      break;
    case 'p':
      args->p = 1;
      break;
    case 't':
      args->t = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options]\n",progname());
  printf("Compute the number of possible alignments between two sequences\n");
  printf("Example: numAl -m 10 -n 20\n");
  printf("Options:\n");
  printf("\t-m <NUM> length of first sequence\n");
  printf("\t-n <NUM> length of second sequence\n");
  printf("\t[-t top-down algorithm; default: bottom up algorithm]\n");
  printf("\t[-p print matrix; default: do not print matrix\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
