/***** pamLog.c ***********************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Jan  6 11:52:39 2015
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "interface.h"
#include "eprintf.h"
#include "pam.h"

void scanFile(FILE *fp, Args *args);
int round2(float x);

int main(int argc, char *argv[]){
  int i;
  char *version;
  Args *args;
  FILE *fp;

  version = "0.7";
  setprogname2("pamLog");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  if(args->numInputFiles == 0){
    fp = stdin;
    scanFile(fp, args);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      scanFile(fp, args);
      fclose(fp);
    }
  }
  free(args);
  free(progname());
  return 0;
}

void scanFile(FILE *fp, Args *args){
  SubstitutionMatrix *matrix;
  int i, j;

  matrix = readSubstitutionMatrix(fp);
  /* take log and round */
  for(i=0;i<matrix->size;i++)
    for(j=0;j<matrix->size;j++)
      matrix->mat[i][j] = round2(log(matrix->mat[i][j])/log(2.0)/args->b);
  outputMatrixInt(stdout,matrix);
}

int round2(float x){
  double ip;

  if(modf(x, &ip) >= 0.5){
    return (int) ++ip;
  }else if(modf(x, &ip) <= -0.5){
    return (int) --ip;
  }else{
    return (int) ip;
  }
}
