/***** pamNormalize.c *****************************
 * Description: Normalize PAM matrix.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Jan  6 11:52:51 2015
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "interface.h"
#include "eprintf.h"
#include "pam.h"
#include "tab.h"

void scanFile(FILE *fp, Args *args, Matrix *aa);

int main(int argc, char *argv[]){
  int i;
  char *version;
  Args *args;
  FILE *fp;
  Matrix *aa;

  version = "0.7";
  setprogname2("pamNormalize");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  fp = efopen(args->a,"r");
  aa = getMatrix(fp,args->f);
  fclose(fp);
  if(args->numInputFiles == 0){
    fp = stdin;
    scanFile(fp, args, aa);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      scanFile(fp, args, aa);
      fclose(fp);
    }
  }
  free(args);
  free(progname());
  return 0;
}

void scanFile(FILE *fp, Args *args, Matrix *aa){
  SubstitutionMatrix *matrix; 
  int i, j;
  double x;

  matrix = readSubstitutionMatrix(fp);
  if(aa->numRows != matrix->size)
    eprintf("main - amino acid frequencies: %d; matrix rows: %d\n",aa->numRows,matrix->size);
  /* matrix normalization */
  for(i=0;i<matrix->size;i++)
    for(j=0;j<aa->numRows;j++)
      matrix->mat[i][j] /= aa->mat[i][0];
  /* make matrix symmetrical */
  for(i=1;i<matrix->size;i++)
    for(j=0;j<i;j++){
      x = (matrix->mat[i][j] + matrix->mat[j][i]) / 2.0;
      matrix->mat[i][j] = x;
      matrix->mat[j][i] = x;
    }
  outputMatrix(stdout,matrix,args->f);
}
