/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  char c;
  char *optString = "hvs:l:g:n:L:";

  args = (Args *)emalloc(sizeof(Args));
  args->l = DEFAULT_L;
  args->L = DEFAULT_LL;
  args->g = DEFAULT_G;
  args->n = DEFAULT_N;
  args->s = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 's':                           /* seed for random number generator */
      args->s = atoi(optarg);
      break;
    case 'n':                           /* number of sequences */
      args->n = atoi(optarg);
      break;
    case 'g':                           /* seed for random number generator */
      args->g = atof(optarg);
      break;
    case 'l':                           /* length of output sequence */
      args->l = atoi(optarg);
      break;
    case 'L':                           /* line length */
      args->L = atoi(optarg);
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options]\n",progname());
  printf("Generate random sequence\n");
  printf("Example: ranseq\n");
  printf("Options:\n");
  printf("\t[-l <NUM> sequence length; default: %d]\n",DEFAULT_L);
  printf("\t[-n <NUM> number of sequences; default: %d]\n",DEFAULT_N);
  printf("\t[-g <NUM> G/C content; default: %.2f]\n",DEFAULT_G);
  printf("\t[-s <NUM> seed for random number generator; default: randomSeed.dat or system clock]\n");
  printf("\t[-L <NUM> line length; default: %d]\n",DEFAULT_LL);
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
