/***** kerrorInterface.c ******************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Mon Dec 20 19:53:47 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include "interface.h"
#include "alignment.h"
#include "sblast.h"
#include "eprintf.h"

Args getArgs(int argc, char *argv[]){
  Args args;
  char c;
  int arg;
  int input;

  args.i = NULL;
  args.j = NULL;
  args.o = NULL;
  args.O = -INT_MAX;
  args.E = -INT_MAX;
  args.w = -INT_MAX;
  args.A = MATCH;
  args.I = MISMATCH;
  args.m = NULL;
  args.s = 20;
  args.t = 50;
  args.k = 1;
  args.p = 0;
  args.a = 1;
  args.l = LINE_LENGTH;
  args.L = 0;
  args.r = 0;
  args.h = 0;
  args.e = 1;
  input = 0;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'i':                           /* first input file */
	args.i = argv[++arg];
	input++;
	args.e = 0;
	break;
      case 'j':                           /* second input file */
	args.j = argv[++arg];
	input++;
	break;
      case 'o':                           /* output file */
	args.o = argv[++arg];
	break;
      case 'O':                           /* gap opening score */
	args.O = atof(argv[++arg]);
	break;
      case 'E':                           /* gap extension score */
	args.E = atoi(argv[++arg]);
	break;
      case 'w':                           /* word length score */
	args.w = atoi(argv[++arg]);
	break;
      case 'A':                           /* match score */
	args.A = atoi(argv[++arg]);
	break;
      case 'I':                           /* mismatch score */
	args.I = atoi(argv[++arg]);
	break;
      case 's':                           /* maximum number idle extension steps */
	args.s = atoi(argv[++arg]);
	break;
      case 't':                           /* minimum score */
	args.t = atoi(argv[++arg]);
	break;
      case 'm':                           /* external score matrix */
	args.m = argv[++arg];
	break;
      case 'p':                           /* protein sequence */
	args.p = 1;
	break;
      case 'l':                           /* line length */
	args.l = atoi(argv[++arg]);
	break;
      case 'L':                           /* print word list */
	args.L = 1;
	break;
      case 'r':                           /* include reverse strand */
	args.r = 1;
	break;
      case 'a':                           /* do not print alignment */
	args.a = 0;
	break;
      case 'k':                           /* maximal number of errors allowed */
	args.k = atoi(argv[++arg]);
	break;
      case 'h':                           /* print help */
	args.h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args.e = 1;
	return args;
      }
      arg++;
  }
  if(args.p && args.m == NULL){
    printf("ERROR [%s]: please enter path for protein score matrix using -m\n",progname());
    exit (-1);
  }
  if(args.p){
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_PROTEIN;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_PROTEIN;
    if(args.w == -INT_MAX)
      args.w = WORDLEN_PROTEIN;
  }else{
    if(args.O == -INT_MAX)
      args.O = GAP_OPENING_DNA;
    if(args.E == -INT_MAX)
      args.E = GAP_EXTENSION_DNA;
    if(args.w == -INT_MAX)
      args.w = WORDLEN_DNA;
  }

  return args;
}

void printUsage(char *version){
  printf("sblast version %s, copyright (c) 2006, Bernhard Haubold\n", version);
  printf("purpose: implement simple, ungapped BLAST algorithm\n");
  printf("reference: Altschul, S. et al. (1990). J. Mol. Biol., 215:403-410.\n"); 
  printf("arguments:  \n");
  printf("\t-i <FILE> read query sequence from FILE\n");
  printf("\t[-j <FILE> read subject sequence from FILE; default: stdin]\n");
  printf("\t[-o <FILE> write output to FILE; default: stdout]\n");
  printf("\t[-w <NUM> word length = NUM; default DNA: %d, default protein: %d]\n",WORDLEN_DNA,WORDLEN_PROTEIN);	     
  printf("\t[-A <NUM> match; default: %d]\n",MATCH);
  printf("\t[-I <NUM> mismatch; default: %d]\n",MISMATCH);	
  printf("\t[-s <NUM> max. idle extension steps; default: %d]\n",20);
  printf("\t[-t <NUM> min. score; default: %d]\n",50);
  printf("\t[-L print word list]\n");
  printf("\t[-h print this help message]\n");
}
