/***** testMeans.c ********************************
 * Description: Compare two means.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Thu Dec 15 10:44:31 2016
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "interface.h"
#include <gsl/gsl_statistics_double.h>
#include "eprintf.h"
#include "tab.h"
#include "tTest.h"
#include "mcTest.h"
#include "gsl_rng.h"

#define MAX_ID_LEN 256

void test(FILE *fp1, FILE *fp2, Args *args){
  char *line, *id1, *id2;
  double *s1, *s2;
  int n1, n2, f, l, i;
  Result *res;
  gsl_rng *ran; 

  s1 = s2 = NULL;
  n1 = n2 = 0;
  l = 0;
  id1 = (char *)emalloc(MAX_ID_LEN);
  id2 = (char *)emalloc(MAX_ID_LEN);
  res = (Result *)emalloc(sizeof(Result));
  while((line = tabGetLine(fp1)) != NULL){
    l++;
    f = tabNfield() - 1;
    if(f > n1)
      s1 = (double *)erealloc(s1, f * sizeof(double));
    n1 = f;
    id1 = strncpy(id1,tabField(0),MAX_ID_LEN);
    for(i=1; i<=n1; i++)
      s1[i-1] = atof(tabField(i));
    line = tabGetLine(fp2);
    if(!line){
      printf("ERROR: unpaired line #%d\n",l);
      exit(-1);
    }
    id2 = strncpy(id2,tabField(0),MAX_ID_LEN);
    if(strcmp(id1,id2)){
      printf("ERROR: %s on line %d paired with %s.\n", id1, l, id2);
      exit(-1);
    }
    f = tabNfield() - 1;
    if(f > n2)
      s2 = (double *)erealloc(s2, f * sizeof(double));
    n2 = f;
    for(i=1; i<=n2; i++)
      s2[i-1] = atof(tabField(i));
    if(args->t == 'm'){
      ran = ini_gsl_rng(args);
      res = mcTest(res, ran, args->i, s1, n1, s2, n2);
      free_gsl_rng(ran, args);
    }else if(args->t == 's')
      res = student(res, s1, n1, s2, n2);
    else if(args->t == 'w')
      res = welch(res, s1, n1, s2, n2);
    printf("%s\t%.3e\t%.3e\t%.3e\n", id1, res->m1, res->m2, res->p);
  }
  free(id1);
  free(id2);
}


int main(int argc, char *argv[]){
  int i, j;
  char *version;
  Args *args;
  FILE *fp1, *fp2;

  version = "0.2";
  setprogname2("testMeans");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  for(i=0;i<args->numInputFiles;i++){
    for(j=i+1;j<args->numInputFiles;j++){
      fp1 = efopen(args->inputFiles[i],"r");
      fp2 = efopen(args->inputFiles[j],"r");
      test(fp1, fp2, args);
      fclose(fp1);
      fclose(fp2);
    }
  }
  free(args);
  free(progname());
  return 0;
}
